#####################################
# Librairie de gestion des fichiers #
#####################################

## Chargement des librairies
catch {global libChargees}
if {! [info exists libChargees]} {
#tk_messageBox -message "n'existe pas" 
set libChargees [list]
}
proc utiliseLibrairie {nomLib} {
  global libChargees

  # On recherche cette librairie dans la liste des librairies dj charges
  set indLib [lsearch $libChargees $nomLib]
  if {$indLib == -1} {
    # Librairie pas encore charge => on la charge !
    source [file join "Lib" "Lib_$nomLib\.tcl"]
    lappend libChargees $nomLib
  }
}

# Lecture du contenu complet d'un fichier
proc lireFichier {nomFichier} {
  set fic [open $nomFichier r]
  set contenu [read $fic]
  close $fic
  return $contenu
}

# Lecture du contenu complet d'un fichier
# retourne une liste dont les lments 
# sont les lignes du fichier  
proc lireFichierDansListe {nomFichier} {
  set fic [open $nomFichier r]
  set liste [list]
  while {[eof $fic] == 0} {
    lappend liste [gets $fic]
  }
  close $fic
  return $liste
}


# cheminRacine = chemin de la racine du file systme
global cheminRacine
set cheminRacine [file normalize /]
# cheminHome = chemin du home-directory de l'utilisateur (sur PPC = "/")
global cheminHome
set cheminHome ""
global PARAM
if {! [file writable "$PARAM(rep)"]} {
  set cheminHome [file normalize ~]
  set ficManuel [file join $cheminHome "Newmagic.progs" "env.ini"]
  if {[file exists $ficManuel]} {
    set nouvRep [lireFichier $ficManuel]
    if {[file writable $nouvRep]} {
      set cheminHome $nouvRep
    }
  }
  set cheminHome [file join $cheminHome "Newmagic.progs" "$PARAM(nomAppli)"]
} else {
  set cheminHome [file normalize "$PARAM(rep)"]
#set cheminHome [file join $cheminHome "$PARAM(nomAppli)"]
}

# Calcul du chemin complet du fichier spcifi :
# si le nom du fichier commence dj par la racine (ou par x:\), on le retourne tel quel
# sinon, on rajoute au dbut : le home directory + "newmagic.progs" + <nom de l'appli>
proc cheminComplet {nomFichier} {
  global cheminRacine
  global cheminHome
  global PARAM

  if {(! [string match "^$cheminRacine\*$" $nomFichier]) && (! [string match "^.:\\\*$" $nomFichier]) && (! [string match "^\.*$" $nomFichier])} {
    set nomFichier [file join $cheminHome $nomFichier]
#msgBox "$cheminRacine\n$cheminHome\n$PARAM(rep)\n$cheminSauv\n$nomFichier\n"
    creerRep [file dirname $nomFichier] ;# on cre le rpertoire s'il n'existe pas dj
  }
  return $nomFichier
}

proc creerRep {nomRep} {
  catch {file mkdir $nomRep} ;# on cre le rpertoire s'il n'existe pas dj
}

# Ecriture de donnes dans un fichier, 
# en spcifiant si l'on ne souhaite pas rajouter une dernire ligne  la fin
proc ecrireFichier {nomFichier donnees {nonewline 0}} {
  set fic [open $nomFichier w]
  if {$nonewline} {
    puts -nonewline $fic "$donnees"
  } else {
    puts $fic "$donnees"
  }
  close $fic
}

# Lecture d'une ligne d'un fichier, au hasard
proc ligneFicHasard {nomFic} {
  set decoupeLignes [lireFichierDansListe $nomFic]
  set nbLignes [llength $decoupeLignes]
  set indiceLigne [expr int(rand()*$nbLignes)]
  return [lindex $decoupeLignes $indiceLigne]
}

# Fonction permettant de lister les fichiers
# verifiant le pattern (ex : *.txt)
# contenus dans le rpertoire indiqu
proc listeFic {pattern {repertoire ""}} {
  if {$repertoire != ""} {
    set sauvRep [pwd] ;# On sauvegarde le rpertoire courant
    cd $repertoire ;# on se place dans le rpertoire indiqu
    set liste [glob -nocomplain $pattern]
    cd $sauvRep ;# On retourne dans le rpertoire sauv
  } else {
    set liste [glob -nocomplain $pattern]
  }
  return [lsort $liste]
}


# Fonction permettant de rcuprer la liste des extensions dans le format utilis par tk_getOpenFile
#  partir de la liste simple des extensions (.ext), et en rajoutant $debLib pour les dbut des libells
proc genererListeExtensions {aPartirDeListe {debLib "Fichiers "}} {
  set listeExt [list]
  foreach ext $aPartirDeListe {
    if {$ext != "*"} {
      if {[string index $ext 0] == "."} {
        set libelle "$debLib[string range $ext 1 end]"
      } else {
        set libelle "$debLib$ext"
      }
    } else {
      set libelle "Tous les fichiers"
    }
    lappend listeExt [list "$libelle" "$ext"]
  }
  return $listeExt
}


# Calculer les diffrences entre 2 fichiers
# Retourne le message (qui peut tre TREEEEES long) indiquant ces diffrences
# ou -1 s'il y a eu une erreur (fichier(s) inexistant(s))
proc diffFichiers {nomFic1 nomFic2} {
  if {[file exists $nomFic1] && [file exists $nomFic2]} {
    set fic1 [lireFichier $nomFic1]
    set fic2 [lireFichier $nomFic2]
    return [diffContenus "$fic1" "$fic2"]
  } else {
    return -1
  }
}

# Calculer les diffrences entre 2 textes
# Retourne le message (qui peut tre TREEEEES long) indiquant ces diffrences
proc diffContenus {fic1 fic2} {
  set fic1 [split $fic1 "\n"]
  set fic2 [split $fic2 "\n"]
    set lst1 $fic1 ;# On copie mais a va bouger
    set lst2 $fic2 ;# On copie mais a va bouger

    set tailleFic1 [llength "$fic1"]
    set tailleFic2 [llength "$fic2"]
    set nbLignesMax $tailleFic1
    if {$nbLignesMax < $tailleFic2} {
      set nbLignesMax $tailleFic2
    }
    
    # On commence par rechercher chaque ligne de fic1 dans fic2 :
    set diff [list]
    set indLigne 1
    foreach ligne $fic1 {
      #set indTrouve [lsearch $lst2 "$ligne"]
      set continuer true
      set i 0
      set indTrouve -1
      while {$continuer} {
        if {$i < [llength $lst2]} {
          if {"$ligne" == "[lindex $lst2 $i]"} {
            set indTrouve $i
            set continuer false
          }
          incr i
        } else {
          set continuer false
        }
      }
      if {$indTrouve >= 0} {
        # Trouv !
  #tk_messageBox -message "$indTrouve"
        if {$indTrouve > 0} {
          # Trouv, mais plus loin... => les lignes prcdentes sont "en plus" dans fic2 :
          set indLigne2 [expr {$tailleFic2 - [llength $lst2]}]
          set tmp [lrange $fic2 $indLigne2 [expr {$indLigne2 + $indTrouve - 1}]]
          incr indLigne2
  #tk_messageBox -message "ligne $indLigne : 
  #$ligne
  #trouve dans fic2  la ligne $indLigne2"
          foreach l $tmp {
            lappend diff "B-$indLigne2\: $l"
            incr indLigne2
          }
        }
        set lst2 [lrange $lst2 [expr {$indTrouve + 1}] end]
      } else {
        # pas trouv... On l'ajoute aux diff :
        lappend diff "A-$indLigne\: $ligne"
      }
      incr indLigne
    }
    set indLigne2 [expr {$tailleFic2 - [llength $lst2] + 1}]
    foreach l $lst2 {
      lappend diff "B-$indLigne2\: $l"
      incr indLigne2
    }

    return [liste2chaine $diff "\n"]
}


### SAUVEGARDES
# Fonction permettant de sauver les variables globales
# dont le nom est compris dans listeVarsGlobales
# dans le fichier ficNom
proc sauverGlobales {ficNom listeVarsGlobales} {
  set contenu ""
  foreach variable $listeVarsGlobales {
    set indParenth [string first "(" $variable]
    if {$indParenth > 0} {
      set variableTab [string range $variable 0 [expr {$indParenth - 1}]]
      global $variableTab
    } else {
      global $variable
    }
    if {[array exists $variable]} {
      foreach nom [array names $variable] {
        set valeur "[subst $[subst $variable]($nom)]"
        append contenu "$variable\($nom\)\n$valeur\n"
      }
    } else {
      catch {
        set valeur "[subst $[subst $variable]]"
        append contenu "$variable\n$valeur\n"
      }
    }
  }
  if {$contenu != ""} {
    ecrireFichier "$ficNom" "$contenu" 1
  }
}
# Fonction permettant de charger les variables
# globales dont les noms et valeurs sont enregistrs
# dans le fichier ficNom
proc chargerGlobales {ficNom} {
  set listeContenu [lireFichierDansListe $ficNom]
  foreach {variable valeur} $listeContenu {
    set indParenth [string first "(" $variable]
    if {$indParenth > 0} {
      set variableTab [string range $variable 0 [expr {$indParenth - 1}]]
      global $variableTab
    } else {
      global $variable
    }
    set $variable $valeur
  }
}


### Logs :
# DEBUG (1), INFO (2), WARNING(3), ERROR(3)
set niveauLog 2
set ficLog [cheminComplet "Logs.txt"]
set formatDateLog "%d/%m/%Y %H:%M:%S"

proc getClock {format} {
  return [clock format [clock second] -format $format]
}

proc logDebug {message} {
  global niveauLog
  global ficLog
  global formatDateLog

  if {$niveauLog <= 1} {
    set fic [open $ficLog a]
    set dateHeure [getClock $formatDateLog]
    puts $fic "$dateHeure - DEBUG : $message"
    close $fic
  }
}

proc logInfo {message} {
  global niveauLog
  global ficLog
  global formatDateLog

  if {$niveauLog <= 2} {
    set fic [open $ficLog a]
    set dateHeure [getClock $formatDateLog]
    puts $fic "$dateHeure - INFO : $message"
    close $fic
  }
}

proc logWarning {message} {
  global niveauLog
  global ficLog
  global formatDateLog

  if {$niveauLog <= 3} {
    set fic [open $ficLog a]
    set dateHeure [getClock $formatDateLog]
    puts $fic "$dateHeure - WARNING : $message"
    close $fic
  }
}

proc logError {message} {
  global ficLog
  global formatDateLog

    set fic [open $ficLog a]
    set dateHeure [getClock $formatDateLog]
    puts $fic "$dateHeure - ERREUR : $message"
    close $fic
}
